// ir2lir.cpp

#ifdef __USAGE
%C - IR to C-Max LIR format converter

   -b <filename>     Build file describing contents of LIR
   -d <filename>     Destination .LIR file. This file will be overwritten
   -i <path>         IR Directory base path

   -Z <debug>        Debug level. 20 is a lot, 30 is too much

File extensions must be specified.

Written by Jay Hogg 04Feb2000.
This program and related source are free for distribution in
a non-commercial environment. No warranties given or implied.
Use at your own risk.

#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
   #include "getopt.h"
#else
   #include <unistd.h>
#endif

#include "irfile.hpp"
#include "irfile-lir.hpp"
#include "irfile-ir.hpp"
#include "version.hpp"

#include "trace.hpp"

struct IRFILE_LIST {
   char        filename[65];
   IR_FILE_IR  *ir_file;
};

IRFILE_LIST irfile_list[64];
int         irfile_list_ct = 0;

/* ----------------------------------------------------------------------- */

int main(int argc, char ** argv)
{
   TRACE trace(10, "main()");

   IR_FILE_LIR    *file_lir;

   int      opt;
   char     *opt_bldfile   = NULL;
   char     *opt_dstfile   = NULL;
   char     *opt_irpath    = NULL;

   printf("IR2LIR Conversion. Version %s\n", VERSION);
   
   // ****** Init Globals
   memset(&irfile_list, 0, sizeof(irfile_list));
   irfile_list_ct = 0;

   // ****** Handle options
   while ((opt = getopt(argc, argv, "b:d:i:Z:")) != -1) {
      switch (opt) {
         case 'b':   opt_bldfile = optarg;         break;
         case 'd':   opt_dstfile = optarg;         break;
         case 'i':   opt_irpath = optarg;          break;
         case 'Z':   TRACE::SetGlobalLevel(atoi(optarg)); break;
      }
   }

   // ****** Sanity Check
   if (!opt_bldfile || !opt_dstfile) {
      printf("Both the build file (-b) and destination file (-d) must be specified.\n");
      return 1;
   }

   // ****** Create the LIR file empty
   file_lir = new IR_FILE_LIR(opt_bldfile);

   // ****** Build the file
   {
      FILE *file_build;
      
      // ****** Open the file
      if ((file_build = fopen(opt_bldfile, "r")) == NULL) {
         perror(opt_bldfile);
         return 1;
      }

      // ****** Read Records - This is a very rigid format
      while (!feof(file_build)) {

         char  buffer[128];
         int   lir_idx;
         char  irfile_name[65];
         char  irfile_code[33];
         int   idx;
         IR_DATA *ir;

         // ****** Read the next line
         memset(buffer, 0, sizeof(buffer));
         if (fgets(buffer, sizeof(buffer)-1, file_build) == NULL)
            continue;

         // ****** Strip trailing garbage
         while (buffer[0] && (buffer[strlen(buffer)-1] <= ' '))
            buffer[strlen(buffer)-1] = 0;

         if ((buffer[0] == ';') || !buffer[0])
            continue;

         // ****** Parse the records
         lir_idx = atoi(strtok(buffer, " "));
         strcpy(irfile_name, strtok(NULL, "#"));
         strcpy(irfile_code, strtok(NULL, " "));
         trace.write(0, "LirIdx=%i File=%s Code=%s", lir_idx, irfile_name, irfile_code);

         // ****** Find the ir file if we loaded it
         for (idx=0; idx<irfile_list_ct; idx++) {
            if (strcmp(irfile_name, irfile_list[idx].filename) == NULL)
               break;
         }

         // ****** Load the file if not found
         if (idx == irfile_list_ct) {
            char tmp[256];
            tmp[0] = 0;
            if (opt_irpath)
               strcat(tmp, opt_irpath);
            sprintf(&tmp[strlen(tmp)], "%s.ir", irfile_name);
            strcpy(irfile_list[idx].filename, irfile_name);
            irfile_list[idx].ir_file = new IR_FILE_IR(irfile_name);
            irfile_list[idx].ir_file->Read(tmp);
            irfile_list_ct++;
         }

         // ****** Get the specified IR
         if ((ir = irfile_list[idx].ir_file->IRFromName(irfile_code)) != NULL) {
            char tmp[32];
            IR_DATA *ir_copy = new IR_DATA(ir);
            sprintf(tmp, "#%i", lir_idx);
            if (file_lir->IRInsert(tmp, ir_copy, lir_idx) != 0) {
               trace.write(-1, "Unable to insert %s#%s @ %i into specified location",
                  irfile_name, irfile_code, lir_idx);
            }
         }
         else {
            trace.write(-1, "Unable to locate %s#%s", irfile_name, irfile_code);
         }

      }

      fclose(file_build);
   }

   // ****** Write the new LIR
   file_lir->Write(opt_dstfile);

   // ****** Cleanup
   {
      int i;

      for (i=0; i<irfile_list_ct; i++)
         delete irfile_list[i].ir_file;

      delete file_lir;
   }

   return 0;
}

/* ----------------------------------------------------------------------- */



