// irfile-ir.cpp

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef WIN32
   #include <unix.h>
   #include <unistd.h>
#endif

#ifdef WIN32
   #define strcasecmp stricmp
   #define strcasencmp strnicmp
#endif

#include "irfile.hpp"
#include "irfile-ir.hpp"

#include "trace.hpp"

/* ----------------------------------------------------------------------- */

IR_FILE_IR::IR_FILE_IR(
   char     *ident)
   : IR_FILE_BASE(ident)
{
}

/* ----------------------------------------------------------------------- */

IR_FILE_IR::~IR_FILE_IR()
{
}

/* ----------------------------------------------------------------------- */

int IR_FILE_IR::Read(
   char     *filename)
{
   TRACE trace(20, "IR_FILE_IR::Read()");
   
   FILE  *file_in;

   char  save_codename[33];
   char  save_desc[33];

   // ****** Open File
   if ((file_in = fopen(filename, "r")) == NULL) {
      trace.write(-1, "Unable to open file '%s'", filename);
      return -1;
   }
                  
   // ****** Scan the lines looking for stuff. NOTE: This is a very
   //        unforgiving scan at the moment...
   while (!feof(file_in)) {

      char  buffer[4096];
      char  *p;

      // ****** Read the line
      memset(buffer, 0, sizeof(buffer));
      if (fgets(buffer, sizeof(buffer)-1, file_in) == NULL)
         continue;

      // ****** Strip trailing garbage
      while (buffer[0] && (buffer[strlen(buffer)-1] <= ' '))
         buffer[strlen(buffer)-1] = 0;

      // ****** Strip leading space
      for (p=buffer; (*p == ' '); p++) ;

      // ****** Skip Comments
      if (*p == ';')
         continue;

      // ****** Snag [codename]
      if (*p == '[') {
         strcpy(save_codename, strtok(++p, "]"));
         trace.write(0, "Begin codename: '%s'", save_codename);
         continue;
      }

      // ****** Description
      if (strcasencmp(p, "Desc", 4) == 0) {
         strtok(p, "\"");
         strcpy(save_desc, strtok(NULL, "\""));
         trace.write(0, "Desc: '%s'", save_desc);
      }

      // ****** ProntoData
      if (strcasencmp(p, "ProntoData", 10) == 0) {
         IR_DATA *ir = new IR_DATA(save_codename, 0, save_desc);
         p = strchr(p, '\"')+1;
         IRInsert(save_codename, ReadIRasPronto(p, ir));
      }

   }

   return trace.ret_int(-1);
}

/* ----------------------------------------------------------------------- */

int IR_FILE_IR::Write(
   char     *filename)
{
   TRACE trace(20, "IR_FILE_IR::Write()");
   FILE  *file_output;
   
   // ****** Open the file
   if ((file_output = fopen(filename, "w")) == NULL) {
      trace.write(-1, "Unable to create file '%s'", filename);
      return -1;
   }

   // ****** Write out a header
   fprintf(file_output, ";\n; %s\n;\n\n", ident);

   // ****** Write out all codes
   {
      int i;
      IR_DATA *ir;

      for (i=0; i<irdata_alloc; i++) {

         if (irdata[i] == NULL)
            continue;

         ir = irdata[i];

         fprintf(file_output, "[%s]\n\n", ir->GetCodename());
         fprintf(file_output, "   Desc \"%s\"\n", ir->GetComment());

         WriteIRasPronto(file_output, ir);
//       WriteIRasPairDoc(file_output, ir);
         
         fprintf(file_output, "\n");

      }

   }

   // ****** Write out footer
   fprintf(file_output, ";\n; End of file\n;\n");

   fclose(file_output);

   return trace.ret_int(-1);
}

/* ----------------------------------------------------------------------- */

void IR_FILE_IR::WriteIRasPronto(
   FILE     *file,
   IR_DATA  *ir)
{
   unsigned short    pdata[1000];
   unsigned short    pdata_ct;

   memset(&pdata, 0, sizeof(pdata));
   pdata_ct = 0;

   if (ir->GetFreq() == 0) {
//    trace.write(-1, "Invalid frequency for [%s]", ir->GetCodename());
      return;
   }

   // ****** Convert IR_DATA to Pronto
   {
      int i;
      unsigned long on_us;
      unsigned long off_us;
      unsigned long factor;

      // ****** Calculate adjustment factor
      factor = 1000000 / ir->GetFreq();
      if (factor == 0) {
         // ... need error
         return;
      }

      // ****** Header Info
      pdata[pdata_ct++] = 0;
      pdata[pdata_ct++] = (unsigned short)(4145146 / ir->GetFreq());
      pdata[pdata_ct++] = (unsigned short)ir->GetBurstPairCount(IR_DATA::BP_TYPE_INITIAL);
      pdata[pdata_ct++] = (unsigned short)ir->GetBurstPairCount(IR_DATA::BP_TYPE_REPEAT);

      // ****** Initial Burst Pairs
      for (i=0; ir->GetBurstPair(IR_DATA::BP_TYPE_INITIAL, &on_us, &off_us, i) == 0; i++) {
         pdata[pdata_ct++] = (unsigned short)(on_us / factor);
         pdata[pdata_ct++] = (unsigned short)(off_us / factor);
      }

      // ****** Repeat Burst Pairs
      for (i=0; ir->GetBurstPair(IR_DATA::BP_TYPE_REPEAT, &on_us, &off_us, i) == 0; i++) {
         pdata[pdata_ct++] = (unsigned short)(on_us / factor);
         pdata[pdata_ct++] = (unsigned short)(off_us / factor);
      }

   }

   // ****** Write Pronto data to file
   {
      int i;

      fprintf(file, "   ProntoData \"");

      for (i=0; i<pdata_ct; i++)
         fprintf(file, "%04x ", pdata[i]);

      fprintf(file, "\"\n");
   }

}

/* ----------------------------------------------------------------------- */

void IR_FILE_IR::WriteIRasPairDoc(
   FILE     *file,
   IR_DATA  *ir)
{
   // ****** Write Documentation
   {
      int i;
      unsigned long on_us;
      unsigned long off_us;
      
      fprintf(file, "   ; Freq %i\n", ir->GetFreq());
      
      fprintf(file, "   ; Initial (%4i) ", ir->GetBurstPairCount(IR_DATA::BP_TYPE_INITIAL));
      for (i=0; ir->GetBurstPair(IR_DATA::BP_TYPE_INITIAL, &on_us, &off_us, i) == 0; i++) {
         fprintf(file, "%5i/%5i|", on_us, off_us);
      }
      fprintf(file, "\n");
      
      fprintf(file, "   ; Repeat  (%4i) ", ir->GetBurstPairCount(IR_DATA::BP_TYPE_REPEAT));
      for (i=0; ir->GetBurstPair(IR_DATA::BP_TYPE_REPEAT, &on_us, &off_us, i) == 0; i++) {
         fprintf(file, "%5i/%5i|", on_us, off_us);
      }
      fprintf(file, "\n\n");

   }  

}

/* ----------------------------------------------------------------------- */

IR_DATA *IR_FILE_IR::ReadIRasPronto(
   char     *buffer,
   IR_DATA  *ir)
{
   TRACE trace(30, "IR_FILE_IR::ReadIRasPronto()");
   unsigned short    pdata[1000];
   unsigned short    pdata_ct;

   memset(&pdata, 0, sizeof(pdata));
   pdata_ct = 0;

   // ****** Convert hex to array
   {
      char *c = strtok(buffer, " \"");

      while (c) {
         pdata[pdata_ct++] = (unsigned short)strtoul(c, NULL, 16);
         c = strtok(NULL, " \"");
      }
   }

   // ****** Handle some "dynamic" stuff for playing
   {
      if (pdata[2] == 0xffff) {
         pdata[2] = ((pdata_ct - 4)/2);
      }
      if (pdata[3] == 0xffff) {
         pdata[3] = ((pdata_ct - 4 - pdata[2])/2);
      }
      
   }
   
   // ****** Sanity check count
   {
      if (pdata_ct != (4 + ((pdata[2] + pdata[3]) * 2))) {
         trace.write(-1, "Pronto sanity check failed on '%s:[%s]'", 
            ident, ir->GetComment());
         return NULL;
      }
   }

   // ****** Convert Frequency - Adjust for known granularity on Pronto
   {
      unsigned long freq;
      switch (pdata[1]) {
         case 0x0067:   freq = 40000;                 break;
         default:       freq = (4145146 / pdata[1]);  break;
      }
      trace.write(20, "Freq = %i", freq);
      ir->SetFreq(freq);
   }

   // ****** Convert array to Burst Pairs
   {
      int i, j;
      unsigned long factor = 1000000 / (4145146 / pdata[1]);

      j = 4;
      for (i=0; i<pdata[2]; i++, j+=2) {
         if (ir->AddBurstPair(IR_DATA::BP_TYPE_INITIAL, 
            pdata[j] * factor, pdata[j+1] * factor) != 0)
         break;
      }

      for (i=0; i<pdata[3]; i++, j+=2) {
         if (ir->AddBurstPair(IR_DATA::BP_TYPE_REPEAT, 
            pdata[j] * factor, pdata[j+1] * factor) != 0)
         break;
      }

      if (j != pdata_ct) {
         trace.write(-1, "Internal error in Pronto data handling");
      }
   }

   return ir;
}

/* ----------------------------------------------------------------------- */

