// irfile-lir.cpp

#define FIX_OFF_SCALING 1

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
   #include <io.h>
#else 
   #include <unistd.h>
#endif

#include "irfile.hpp"
#include "irfile-lir.hpp"

#include "trace.hpp"

#ifndef O_BINARY
   #define O_BINARY  0
#endif

/* ----------------------------------------------------------------------- */

IR_FILE_LIR::IR_FILE_LIR(
   char     *ident)
   : IR_FILE_BASE(ident)
{
}

/* ----------------------------------------------------------------------- */

IR_FILE_LIR::~IR_FILE_LIR()
{
}

/* ----------------------------------------------------------------------- */

int IR_FILE_LIR::Read(
   char     *filename,
   int      maxread)
{
   TRACE trace(10, "IR_FILE_LIR::Read()");
   int      fd, count;
   LIR_DS   lir;

   // ****** Open the file
   if ((fd = open(filename, O_RDONLY|O_BINARY)) == -1) {
      trace.write(-1, "Unable to open '%s'", filename);
      return -1;
   }

   // ****** Process all the records
   count = 0;
   while (1) {

      unsigned long freq_hz;
      char     tmp[17];
      IR_DATA  *irdata;

      // ****** Read the next record
      if (read(fd, &lir, sizeof(lir)) != sizeof(lir))
         break;

      trace.write(0, "Processing record %i (0x%04x)", count, count * sizeof(lir));

      // ****** Calculate Frequency in HZ
      freq_hz = ((2304 * 2000) / (lir.freq_mod + 1));
      trace.write(0, "Frequency = %i", freq_hz);

      // ****** Build Object Identifier
      sprintf(tmp, "#%i", count);

      // ****** Build Object
      irdata = new IR_DATA(tmp, freq_hz, tmp);

      // ****** Populate Burst Pairs - Read the docs because this gets
      //        complicated with high-bit indicating polarity and counts > 126
      //        are continued on the next byte in LIR but must be merged for
      //        the absolute format. There are also scaling issues due to
      //        ISR latency and capacitance so the calcs aren't real
      //        obvious.
      {
         unsigned long  bp_on;
         unsigned long  bp_off;
         unsigned long  bp_max;
         int i;

         i = 0;
         bp_max = 0;

         while (i < sizeof(lir.data)) {

            bp_on = bp_off = 0;

            // ****** Count ON Cycles - we always start "on"
            while (((lir.data[i] & 0x80) == 0x80) && (i < sizeof(lir.data))) {
               bp_on += (lir.data[i] & 0x7f);
               i++;
            }

            // ****** Count OFF Cycles - On is *always* followed by off
            while (((lir.data[i] & 0x80) == 0x00) && (i < sizeof(lir.data))) {
               bp_off += (lir.data[i] & 0x7f);
               i++;
            }

            // ****** Adjust for sample rate, ISR latency, and capacitance
            if (bp_on > 2) bp_on -= 2;    // Capacitance
            bp_off += 2;                  // Capacitance
            bp_on  *= (35 + 5);           // Sample rate + latency
            bp_off *= (35 + 5);           // Sample rate + latency

            // ****** Add to list
            irdata->AddBurstPair(IR_DATA::BP_TYPE_INITIAL, bp_on, bp_off);

#ifdef USE_GUESS
            // ****** Try to guess end based on inter-code guard length
            if (bp_max && (bp_off > (bp_max * 2))) {
               trace.write(0, "WAG - inter-code guard on off cycle");
               break;
            }
#endif

            // ***** Store longest burst pair
            if (bp_on + bp_off > bp_max)
               bp_max = bp_on + bp_off;
         }
      }

      IRInsert(tmp, irdata, count);

      count++;

      if ((maxread > 0) && (count == maxread))
         break;
   }

   close(fd);

   trace.write(0, "%i records read", count);

   return 0;
}

/* ----------------------------------------------------------------------- */

int IR_FILE_LIR::Read(
   char     *filename)
{
   return (IR_FILE_LIR::Read(filename, -1));
}

/* ----------------------------------------------------------------------- */

int IR_FILE_LIR::Write(
   char     *filename)
{
   TRACE trace(10, "IR_FILE_LIR::Write()");
   int      fd, i, max_idx;
   LIR_DS   lir;

   // ****** Count the records to write
   for (max_idx=0, i=0; i<irdata_alloc; i++) {
      if (irdata[i])
         max_idx = i;
   }
   trace.write(0, "Max index @ %i", max_idx);

   // ****** Create the file
   if ((fd = open(filename, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0777)) == -1) {
      trace.write(-1, "Unable to create file '%s'", filename);
      return -1;
   }

   // ****** Process all the records
   for (i=0; i<max_idx+1; i++) {

      IR_DATA *ir = irdata[i];

      trace.write(0, "Writing record %i", i);

      // ****** Clear LIR to a "clean" entry
      memset(&lir, 0, sizeof(lir));
      memset(&lir.data, 0, sizeof(lir.data));
      
      // ****** If we have an entry then build the LIR
      if (ir) {

         IR_DATA::BP_TYPE type;
         unsigned long  on_us;
         unsigned long  off_us;
         int   bpseq;
         int   lirofs;

         // ****** Calculate Frequency
         lir.freq_mod = (unsigned char)(((2304 * 2000) / ir->GetFreq()) - 1);

         // ****** Store Comment
         strncpy(lir.comment, ir->GetComment(), sizeof(lir.comment));

         // ****** If we have an "initial" use initial else use repeat
         if (ir->GetBurstPair(IR_DATA::BP_TYPE_INITIAL, &on_us, &off_us, 0) == 0)
            type = IR_DATA::BP_TYPE_INITIAL;
         else
            type = IR_DATA::BP_TYPE_REPEAT;

         // ****** Encode Burst Pairs
         for (lirofs = 0, bpseq = 0;
              (ir->GetBurstPair(type, &on_us, &off_us, bpseq) == 0);
              bpseq++) {

            // ****** Adjust for sample rate, ISR Latency, and capacitance
            //        on Ocelot.

            if (on_us % 40)
               on_us += 40;
            if (off_us % 40)
               off_us += 40;
            on_us  /= (35 + 5);        // Sample rate + latency
            off_us /= (35 + 5);        // Sample rate + latency
            on_us  += 2;               // Capacitance
            if (off_us > 2)            // Capacitance
               off_us -= 2;

            // ****** Encode "On"
            while (on_us && (lirofs < sizeof(lir.data))) {
               if (on_us >= 126) {
                  lir.data[lirofs++] = 0x80 | 126;
                  on_us -= 126;
               }
               else {
                  lir.data[lirofs++] = (unsigned char)(0x80 | on_us);
                  on_us = 0;
               }
            }

            // ****** Encode "Off"
            while (off_us && (lirofs < sizeof(lir.data))) {
               if (off_us >= 126) {
                  lir.data[lirofs++] = 0x00 | 126;
                  off_us -= 126;
               }
               else {
                  lir.data[lirofs++] = (unsigned char)(0x00 | off_us);
                  off_us = 0;
               }
            }

            if (lirofs == sizeof(lir.data)) {
               trace.write(-1, "Overflow in LIR data field. Index = %i", i);
               break;
            }
         }

         // ...
      }

      // ****** Write the entry
      if (write(fd, &lir, sizeof(lir)) != sizeof(lir)) 
         trace.write(-1, "Error writing record");

   }

   close(fd);

   return -1;
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
   
