// irfile.cpp

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef WIN32
   #include <unix.h>
#endif

#ifdef WIN32
   #define strcasecmp stricmp
   #define strcasencmp strnicmp
#endif

#include "irfile.hpp"
#include "trace.hpp"

/* ----------------------------------------------------------------------- */

IR_FILE_BASE::IR_FILE_BASE(
   char     *ident_in)
{
   TRACE trace(10, "IR_FILE_BASE::IR_FILE_BASE()");

   // ****** Save Identifier
   memset(ident, 0, sizeof(ident));
   strncpy(ident, ident_in, sizeof(ident)-1);

   // ****** Allocate array of pointers and init to NULL
   irdata_alloc = MAX_CODENAME;
   irdata = (IR_DATA **)malloc(sizeof(IR_DATA *) * irdata_alloc);
   memset(irdata, 0, sizeof(IR_DATA *) * irdata_alloc);
}

/* ----------------------------------------------------------------------- */

IR_FILE_BASE::~IR_FILE_BASE()
{
   TRACE trace(10, "IR_FILE_BASE::~IR_FILE_BASE()");
   Clear();
}

/* ----------------------------------------------------------------------- */

int IR_FILE_BASE::Clear()
{
   TRACE trace(10, "IR_FILE_BASE::Clear()");
   int i;

   // ****** Free IR_DATA entries
   for (i=0; i<irdata_alloc; i++) {
      if (irdata[i]) 
         delete irdata[i];
   }
      
   return 0;
}

/* ----------------------------------------------------------------------- */

IR_DATA *IR_FILE_BASE::IRFromName(
   char     *codename)
{
   TRACE trace(20, "IR_FILE_BASE::IRFromName()");
   int i;

   // ****** Do lookup - PTR if found, NULL otherwise
   if ((i = IRFind(codename)) != -1)
      return irdata[i];
      
   return NULL;
}

/* ----------------------------------------------------------------------- */

IR_DATA *IR_FILE_BASE::IRFromIdx(
   int      idx)
{
   TRACE trace(20, "IR_FILE_BASE::IRFromIdx()");

   if ((idx >=0) && (idx < MAX_CODENAME) && irdata[idx])
      return irdata[idx];

   return NULL;

}

/* ----------------------------------------------------------------------- */

int IR_FILE_BASE::IRInsert(
   char     *codename,
   IR_DATA  *data,
   int      idx)
{
   TRACE trace(10, "IR_FILE_BASE::IRInsert()");
   int i;

   // ****** Make sure it doesn't exist
   if (IRFind(codename) != -1) {
      trace.write(-1, "%s [%s] already exists", ident, codename);
      return -1;
   }

   // ****** Make sure we have a place to put it
   if (idx == -1) {
      if ((i = IRFind(NULL)) == -1) {
         trace.write(-1, "%s [%s] code table full", ident, codename);
         return -1;
      }
   } else {
      if (irdata[idx]) {
         trace.write(-1, "%s [%s] index %i already populated", ident, codename);
         return -1;
      }
      i = idx;
   }

   // ****** Store It
   trace.write(0, "Stored @ %i", i);
   irdata[i] = data;

   return 0;   
}

/* ----------------------------------------------------------------------- */

int IR_FILE_BASE::IRRemove(
   char     *codename)
{
   TRACE trace(10, "IR_FILE_BASE::IRRemove()");
   int   i;

   // ****** Get the index
   if ((i = IRFind(codename)) == -1) {
      trace.write(-1, "%s [%s] not found to remove", ident, codename);
      return -1;
   }

   // ****** Delete the data
   delete irdata[i];
   irdata[i] = NULL;

   return 0;
}

/* ----------------------------------------------------------------------- */

int IR_FILE_BASE::IRFind(
   char     *codename)
{
   TRACE trace(20, "IR_FILE_BASE::IRFind()");
   int i;

   // ****** Lazy - scan the array
   for (i=0; i<irdata_alloc; i++) {

      // ****** Handle NULL - Looking for empty spot
      if (!codename && !irdata[i])
         return i;

      // ****** Handle Match
      if (codename && irdata[i] && irdata[i]->IsCodename(codename))
         return i;
   }

   return -1;
}

/* ----------------------------------------------------------------------- */

int IR_FILE_BASE::IsIdent(
   char     *ident_in)
{
   return (strcmp(ident_in, ident) == NULL ? 1 : 0);
}

/* ----------------------------------------------------------------------- */

IR_DATA::IR_DATA(
   char     *codename_in,
   unsigned long freq_hz_in,
   char     *comment_in)
{
   strcpy(codename, codename_in);
   freq_hz = freq_hz_in;
   strcpy(comment, comment_in);

   memset(&ir_initial, 0, sizeof(ir_initial));
   memset(&ir_repeat, 0, sizeof(ir_repeat));
}

/* ----------------------------------------------------------------------- */

IR_DATA::IR_DATA(
   IR_DATA  *data)
{
   strcpy(codename, data->codename);
   strcpy(comment, data->comment);
   freq_hz = data->freq_hz;
   memcpy(&ir_initial, &data->ir_initial, sizeof(ir_initial));
   memcpy(&ir_repeat, &data->ir_repeat, sizeof(ir_repeat));
}

/* ----------------------------------------------------------------------- */

IR_DATA::~IR_DATA()
{
}

/* ----------------------------------------------------------------------- */

IR_DATA *IR_DATA::operator=(IR_DATA &in)
{
   strcpy(codename, in.codename);
   strcpy(comment, in.comment);
   freq_hz = in.freq_hz;
   memcpy(&ir_initial, &in.ir_initial, sizeof(ir_initial));
   memcpy(&ir_repeat, &in.ir_repeat, sizeof(ir_repeat));

   return this;
}

/* ----------------------------------------------------------------------- */

int IR_DATA::IsCodename(
   char     *codename_in)
{
   return (strcasecmp(codename_in, codename) == 0 ? 1 : 0);
}

/* ----------------------------------------------------------------------- */

int IR_DATA::AddBurstPair(
   IR_DATA::BP_TYPE type,
   unsigned long  on_us,
   unsigned long  off_us)
{
   TRACE trace(30, "AddBurstPair()");

   IR_DS *p = NULL;

   trace.write(20, "BurstPair(%i %i %i)", type, on_us, off_us);

   // ****** Figure out where we put it...
   switch (type) {
      case BP_TYPE_INITIAL:  p = &ir_initial;   break;
      case BP_TYPE_REPEAT:   p = &ir_repeat;    break;
   }

   // ****** Make sure we have space
   if (p->count >= (sizeof(p->on_us)/sizeof(p->on_us[0]))) {
      trace.write(-1, "Burst pair overflow on [%s]", codename);
      return -1;
   }
   
   // ****** Store it
   p->on_us[p->count]   = on_us;
   p->off_us[p->count]  = off_us;
   p->count++;

   return 0;
}

/* ----------------------------------------------------------------------- */

int IR_DATA::GetBurstPair(
   BP_TYPE        type,
   unsigned long  *on_us,
   unsigned long  *off_us,
   int            idx)
{
   TRACE trace(30, "GetBurstPair()");

   IR_DS *p = NULL;

   // ****** Figure out where to get it from
   switch (type) {
      case BP_TYPE_INITIAL:  p = &ir_initial;   break;
      case BP_TYPE_REPEAT:   p = &ir_repeat;    break;
   }

   // ****** Make sure we have that many
   if (idx >= p->count)
      return -1;

   // ****** Return the data
   *on_us = p->on_us[idx];
   *off_us = p->off_us[idx];

   trace.write(20, "BurstPair(%i %i %i)", type, *on_us, *off_us);

   return 0;
}

/* ----------------------------------------------------------------------- */

int IR_DATA::Merge(
   IR_DATA::BP_TYPE  type_dest,
   IR_DATA::BP_TYPE  type_src,
   IR_DATA           *data_src)
{
   TRACE trace(10, "IR_DATA::Merge()");

   int i;
   IR_DS    *src;
   IR_DS    *dest;

   // ****** Check for frequency compatibility
   if (data_src->freq_hz != freq_hz) {
      trace.write(-1, "%s and %s can not merge - frequency conflict",
         data_src->codename, codename);
      return -1;
   }

   // ****** Source Data Point
   switch (type_src) {
      case BP_TYPE_INITIAL:   src = &data_src->ir_initial;  break;
      case BP_TYPE_REPEAT:    src = &data_src->ir_repeat;   break;
   }

   // ****** Destination Data Point
   switch (type_dest) {
      case BP_TYPE_INITIAL:   dest = &ir_initial;  break;
      case BP_TYPE_REPEAT:    dest = &ir_repeat;   break;
   }

   // ****** Merge Data
   for (i=0; i<src->count; i++) {
      dest->on_us[dest->count] = src->on_us[i];
      dest->off_us[dest->count] = src->off_us[i];
      dest->count++;
      // ... need to watch overruns!
   }

   return 0;
}

/* ----------------------------------------------------------------------- */

int IR_DATA::GetBurstPairCount(
   BP_TYPE        type)
{
   IR_DS *p = NULL;

   // ****** Figure out where to get it from
   switch (type) {
      case BP_TYPE_INITIAL:  p = &ir_initial;   break;
      case BP_TYPE_REPEAT:   p = &ir_repeat;    break;
   }

   return p->count;
}

/* ----------------------------------------------------------------------- */

